import os
import tempfile

__version__ = '2.13.6'


__BUCKET_NAME__= 'carol-internal'
__TEMP_STORAGE__ = os.path.join(tempfile.gettempdir(),'carolina/cache')

__CONNECTOR_PYCAROL__ = 'f9953f6645f449baaccd16ab462f9b64'

