import unittest


class TestValidateCarolValidationRules(unittest.TestCase):
    pass


class TestValidateTypes(unittest.TestCase):
    def test_validate_string(self):
        # TODO
        pass

    def test_validate_boolean(self):
        # TODO
        pass

    def test_validate_date(self):
        # TODO
        pass

    def test_validate_long(self):
        # TODO
        pass

    def test_validate_double(self):
        # TODO
        pass

    def test_validate_binary(self):
        # TODO
        pass

    def test_validate_object(self):
        # TODO
        pass

    def test_validate_geopoint(self):
        # TODO
        pass
